@@ -1,8 +1,10 @@
+# mypy: allow-untyped-defs
 """local path implementation."""
 
 from __future__ import annotations
 
 import atexit
+from collections.abc import Callable
 from contextlib import contextmanager
 import fnmatch
 import importlib.util
@@ -22,7 +24,6 @@ from stat import S_ISLNK
 from stat import S_ISREG
 import sys
 from typing import Any
-from typing import Callable
 from typing import cast
 from typing import Literal
 from typing import overload
@@ -160,15 +161,13 @@ class Visitor:
         )
         if not self.breadthfirst:
             for subdir in dirs:
-                for p in self.gen(subdir):
-                    yield p
+                yield from self.gen(subdir)
         for p in self.optsort(entries):
             if self.fil is None or self.fil(p):
                 yield p
         if self.breadthfirst:
             for subdir in dirs:
-                for p in self.gen(subdir):
-                    yield p
+                yield from self.gen(subdir)
 
 
 class FNMatcher:
@@ -205,12 +204,10 @@ class Stat:
     if TYPE_CHECKING:
 
         @property
-        def size(self) -> int:
-            ...
+        def size(self) -> int: ...
 
         @property
-        def mtime(self) -> float:
-            ...
+        def mtime(self) -> float: ...
 
     def __getattr__(self, name: str) -> Any:
         return getattr(self._osstatresult, "st_" + name)
@@ -225,7 +222,7 @@ class Stat:
             raise NotImplementedError("XXX win32")
         import pwd
 
-        entry = error.checked_call(pwd.getpwuid, self.uid)  # type:ignore[attr-defined]
+        entry = error.checked_call(pwd.getpwuid, self.uid)  # type:ignore[attr-defined,unused-ignore]
         return entry[0]
 
     @property
@@ -235,7 +232,7 @@ class Stat:
             raise NotImplementedError("XXX win32")
         import grp
 
-        entry = error.checked_call(grp.getgrgid, self.gid)  # type:ignore[attr-defined]
+        entry = error.checked_call(grp.getgrgid, self.gid)  # type:ignore[attr-defined,unused-ignore]
         return entry[0]
 
     def isdir(self):
@@ -253,7 +250,7 @@ def getuserid(user):
     import pwd
 
     if not isinstance(user, int):
-        user = pwd.getpwnam(user)[2]  # type:ignore[attr-defined]
+        user = pwd.getpwnam(user)[2]  # type:ignore[attr-defined,unused-ignore]
     return user
 
 
@@ -261,7 +258,7 @@ def getgroupid(group):
     import grp
 
     if not isinstance(group, int):
-        group = grp.getgrnam(group)[2]  # type:ignore[attr-defined]
+        group = grp.getgrnam(group)[2]  # type:ignore[attr-defined,unused-ignore]
     return group
 
 
@@ -318,7 +315,7 @@ class LocalPath:
         def readlink(self) -> str:
             """Return value of a symbolic link."""
             # https://github.com/python/mypy/issues/12278
-            return error.checked_call(os.readlink, self.strpath)  # type: ignore[arg-type,return-value]
+            return error.checked_call(os.readlink, self.strpath)  # type: ignore[arg-type,return-value,unused-ignore]
 
         def mklinkto(self, oldname):
             """Posix style hard link to another name."""
@@ -435,7 +432,7 @@ class LocalPath:
         """Return a string which is the relative part of the path
         to the given 'relpath'.
         """
-        if not isinstance(relpath, (str, LocalPath)):
+        if not isinstance(relpath, str | LocalPath):
             raise TypeError(f"{relpath!r}: not a string or path object")
         strrelpath = str(relpath)
         if strrelpath and strrelpath[-1] != self.sep:
@@ -452,7 +449,7 @@ class LocalPath:
 
     def ensure_dir(self, *args):
         """Ensure the path joined with args is a directory."""
-        return self.ensure(*args, **{"dir": True})
+        return self.ensure(*args, dir=True)
 
     def bestrelpath(self, dest):
         """Return a string which is a relative path from self
@@ -655,12 +652,12 @@ class LocalPath:
         if not kw:
             obj.strpath = self.strpath
             return obj
-        drive, dirname, basename, purebasename, ext = self._getbyspec(
+        drive, dirname, _basename, purebasename, ext = self._getbyspec(
             "drive,dirname,basename,purebasename,ext"
         )
         if "basename" in kw:
             if "purebasename" in kw or "ext" in kw:
-                raise ValueError("invalid specification %r" % kw)
+                raise ValueError(f"invalid specification {kw!r}")
         else:
             pb = kw.setdefault("purebasename", purebasename)
             try:
@@ -706,7 +703,7 @@ class LocalPath:
                     elif name == "ext":
                         res.append(ext)
                     else:
-                        raise ValueError("invalid part specification %r" % name)
+                        raise ValueError(f"invalid part specification {name!r}")
         return res
 
     def dirpath(self, *args, **kwargs):
@@ -757,15 +754,11 @@ class LocalPath:
         if ensure:
             self.dirpath().ensure(dir=1)
         if encoding:
-            # Using type ignore here because of this error:
-            # error: Argument 1 has incompatible type overloaded function;
-            #   expected "Callable[[str, Any, Any], TextIOWrapper]"  [arg-type]
-            # Which seems incorrect, given io.open supports the given argument types.
             return error.checked_call(
                 io.open,
                 self.strpath,
                 mode,
-                encoding=encoding,  # type:ignore[arg-type]
+                encoding=encoding,
             )
         return error.checked_call(open, self.strpath, mode)
 
@@ -841,7 +834,7 @@ class LocalPath:
     def copy(self, target, mode=False, stat=False):
         """Copy path to target.
 
-        If mode is True, will copy copy permission from path to target.
+        If mode is True, will copy permission from path to target.
         If stat is True, copy permission, last modification
         time, last access time, and flags from path to target.
         """
@@ -966,12 +959,10 @@ class LocalPath:
             return p
 
     @overload
-    def stat(self, raising: Literal[True] = ...) -> Stat:
-        ...
+    def stat(self, raising: Literal[True] = ...) -> Stat: ...
 
     @overload
-    def stat(self, raising: Literal[False]) -> Stat | None:
-        ...
+    def stat(self, raising: Literal[False]) -> Stat | None: ...
 
     def stat(self, raising: bool = True) -> Stat | None:
         """Return an os.stat() tuple."""
@@ -1033,7 +1024,7 @@ class LocalPath:
         return self.stat().atime
 
     def __repr__(self):
-        return "local(%r)" % self.strpath
+        return f"local({self.strpath!r})"
 
     def __str__(self):
         """Return string representation of the Path."""
@@ -1054,7 +1045,7 @@ class LocalPath:
     def pypkgpath(self):
         """Return the Python package path by looking for the last
         directory upwards which still contains an __init__.py.
-        Return None if a pkgpath can not be determined.
+        Return None if a pkgpath cannot be determined.
         """
         pkgpath = None
         for parent in self.parts(reverse=True):
@@ -1105,9 +1096,7 @@ class LocalPath:
                 modname = self.purebasename
             spec = importlib.util.spec_from_file_location(modname, str(self))
             if spec is None or spec.loader is None:
-                raise ImportError(
-                    f"Can't find module {modname} at location {str(self)}"
-                )
+                raise ImportError(f"Can't find module {modname} at location {self!s}")
             mod = importlib.util.module_from_spec(spec)
             spec.loader.exec_module(mod)
             return mod
@@ -1279,13 +1268,7 @@ class LocalPath:
 
         if rootdir is None:
             rootdir = cls.get_temproot()
-        # Using type ignore here because of this error:
-        # error: Argument 1 has incompatible type overloaded function; expected "Callable[[str], str]"  [arg-type]
-        # Which seems incorrect, given tempfile.mkdtemp supports the given argument types.
-        path = error.checked_call(
-            tempfile.mkdtemp,
-            dir=str(rootdir),  # type:ignore[arg-type]
-        )
+        path = error.checked_call(tempfile.mkdtemp, dir=str(rootdir))
         return cls(path)
 
     @classmethod
