@@ -1,25 +1,27 @@
+# mypy: allow-untyped-defs
 """Record warnings during test function execution."""
 
+from __future__ import annotations
+
+from collections.abc import Callable
+from collections.abc import Generator
+from collections.abc import Iterator
 from pprint import pformat
 import re
 from types import TracebackType
 from typing import Any
-from typing import Callable
 from typing import final
-from typing import Generator
-from typing import Iterator
-from typing import List
-from typing import Optional
 from typing import overload
-from typing import Pattern
-from typing import Tuple
-from typing import Type
+from typing import TYPE_CHECKING
 from typing import TypeVar
-from typing import Union
+
+
+if TYPE_CHECKING:
+    from typing_extensions import Self
+
 import warnings
 
 from _pytest.deprecated import check_ispytest
-from _pytest.deprecated import WARNS_NONE_ARG
 from _pytest.fixtures import fixture
 from _pytest.outcomes import Exit
 from _pytest.outcomes import fail
@@ -29,11 +31,10 @@ T = TypeVar("T")
 
 
 @fixture
-def recwarn() -> Generator["WarningsRecorder", None, None]:
+def recwarn() -> Generator[WarningsRecorder]:
     """Return a :class:`WarningsRecorder` instance that records all warnings emitted by test functions.
 
-    See https://docs.pytest.org/en/latest/how-to/capture-warnings.html for information
-    on warning categories.
+    See :ref:`warnings` for information on warning categories.
     """
     wrec = WarningsRecorder(_ispytest=True)
     with wrec:
@@ -43,21 +44,17 @@ def recwarn() -> Generator["WarningsRecorder", None, None]:
 
 @overload
 def deprecated_call(
-    *, match: Optional[Union[str, Pattern[str]]] = ...
-) -> "WarningsRecorder":
-    ...
+    *, match: str | re.Pattern[str] | None = ...
+) -> WarningsRecorder: ...
 
 
 @overload
-def deprecated_call(  # noqa: F811
-    func: Callable[..., T], *args: Any, **kwargs: Any
-) -> T:
-    ...
+def deprecated_call(func: Callable[..., T], *args: Any, **kwargs: Any) -> T: ...
 
 
-def deprecated_call(  # noqa: F811
-    func: Optional[Callable[..., Any]] = None, *args: Any, **kwargs: Any
-) -> Union["WarningsRecorder", Any]:
+def deprecated_call(
+    func: Callable[..., Any] | None = None, *args: Any, **kwargs: Any
+) -> WarningsRecorder | Any:
     """Assert that code produces a ``DeprecationWarning`` or ``PendingDeprecationWarning`` or ``FutureWarning``.
 
     This function can be used as a context manager::
@@ -83,7 +80,7 @@ def deprecated_call(  # noqa: F811
     """
     __tracebackhide__ = True
     if func is not None:
-        args = (func,) + args
+        args = (func, *args)
     return warns(
         (DeprecationWarning, PendingDeprecationWarning, FutureWarning), *args, **kwargs
     )
@@ -91,32 +88,30 @@ def deprecated_call(  # noqa: F811
 
 @overload
 def warns(
-    expected_warning: Union[Type[Warning], Tuple[Type[Warning], ...]] = ...,
+    expected_warning: type[Warning] | tuple[type[Warning], ...] = ...,
     *,
-    match: Optional[Union[str, Pattern[str]]] = ...,
-) -> "WarningsChecker":
-    ...
+    match: str | re.Pattern[str] | None = ...,
+) -> WarningsChecker: ...
 
 
 @overload
-def warns(  # noqa: F811
-    expected_warning: Union[Type[Warning], Tuple[Type[Warning], ...]],
+def warns(
+    expected_warning: type[Warning] | tuple[type[Warning], ...],
     func: Callable[..., T],
     *args: Any,
     **kwargs: Any,
-) -> T:
-    ...
+) -> T: ...
 
 
-def warns(  # noqa: F811
-    expected_warning: Union[Type[Warning], Tuple[Type[Warning], ...]] = Warning,
+def warns(
+    expected_warning: type[Warning] | tuple[type[Warning], ...] = Warning,
     *args: Any,
-    match: Optional[Union[str, Pattern[str]]] = None,
+    match: str | re.Pattern[str] | None = None,
     **kwargs: Any,
-) -> Union["WarningsChecker", Any]:
+) -> WarningsChecker | Any:
     r"""Assert that code raises a particular class of warning.
 
-    Specifically, the parameter ``expected_warning`` can be a warning class or sequence
+    Specifically, the parameter ``expected_warning`` can be a warning class or tuple
     of warning classes, and the code inside the ``with`` block must issue at least one
     warning of that class or classes.
 
@@ -172,7 +167,7 @@ def warns(  # noqa: F811
             return func(*args[1:], **kwargs)
 
 
-class WarningsRecorder(warnings.catch_warnings):  # type:ignore[type-arg]
+class WarningsRecorder(warnings.catch_warnings):
     """A context manager to record raised warnings.
 
     Each recorded warning is an instance of :class:`warnings.WarningMessage`.
@@ -187,21 +182,20 @@ class WarningsRecorder(warnings.catch_warnings):  # type:ignore[type-arg]
 
     def __init__(self, *, _ispytest: bool = False) -> None:
         check_ispytest(_ispytest)
-        # Type ignored due to the way typeshed handles warnings.catch_warnings.
-        super().__init__(record=True)  # type: ignore[call-arg]
+        super().__init__(record=True)
         self._entered = False
-        self._list: List[warnings.WarningMessage] = []
+        self._list: list[warnings.WarningMessage] = []
 
     @property
-    def list(self) -> List["warnings.WarningMessage"]:
+    def list(self) -> list[warnings.WarningMessage]:
         """The list of recorded warnings."""
         return self._list
 
-    def __getitem__(self, i: int) -> "warnings.WarningMessage":
+    def __getitem__(self, i: int) -> warnings.WarningMessage:
         """Get a recorded warning by index."""
         return self._list[i]
 
-    def __iter__(self) -> Iterator["warnings.WarningMessage"]:
+    def __iter__(self) -> Iterator[warnings.WarningMessage]:
         """Iterate through the recorded warnings."""
         return iter(self._list)
 
@@ -209,12 +203,12 @@ class WarningsRecorder(warnings.catch_warnings):  # type:ignore[type-arg]
         """The number of recorded warnings."""
         return len(self._list)
 
-    def pop(self, cls: Type[Warning] = Warning) -> "warnings.WarningMessage":
+    def pop(self, cls: type[Warning] = Warning) -> warnings.WarningMessage:
         """Pop the first recorded warning which is an instance of ``cls``,
         but not an instance of a child class of any other match.
         Raises ``AssertionError`` if there is no match.
         """
-        best_idx: Optional[int] = None
+        best_idx: int | None = None
         for i, w in enumerate(self._list):
             if w.category == cls:
                 return self._list.pop(i)  # exact match, stop looking
@@ -232,9 +226,9 @@ class WarningsRecorder(warnings.catch_warnings):  # type:ignore[type-arg]
         """Clear the list of recorded warnings."""
         self._list[:] = []
 
-    # Type ignored because it doesn't exactly warnings.catch_warnings.__enter__
-    # -- it returns a List but we only emulate one.
-    def __enter__(self) -> "WarningsRecorder":  # type: ignore
+    # Type ignored because we basically want the `catch_warnings` generic type
+    # parameter to be ourselves but that is not possible(?).
+    def __enter__(self) -> Self:  # type: ignore[override]
         if self._entered:
             __tracebackhide__ = True
             raise RuntimeError(f"Cannot enter {self!r} twice")
@@ -247,9 +241,9 @@ class WarningsRecorder(warnings.catch_warnings):  # type:ignore[type-arg]
 
     def __exit__(
         self,
-        exc_type: Optional[Type[BaseException]],
-        exc_val: Optional[BaseException],
-        exc_tb: Optional[TracebackType],
+        exc_type: type[BaseException] | None,
+        exc_val: BaseException | None,
+        exc_tb: TracebackType | None,
     ) -> None:
         if not self._entered:
             __tracebackhide__ = True
@@ -266,10 +260,8 @@ class WarningsRecorder(warnings.catch_warnings):  # type:ignore[type-arg]
 class WarningsChecker(WarningsRecorder):
     def __init__(
         self,
-        expected_warning: Optional[
-            Union[Type[Warning], Tuple[Type[Warning], ...]]
-        ] = Warning,
-        match_expr: Optional[Union[str, Pattern[str]]] = None,
+        expected_warning: type[Warning] | tuple[type[Warning], ...] = Warning,
+        match_expr: str | re.Pattern[str] | None = None,
         *,
         _ispytest: bool = False,
     ) -> None:
@@ -277,15 +269,14 @@ class WarningsChecker(WarningsRecorder):
         super().__init__(_ispytest=True)
 
         msg = "exceptions must be derived from Warning, not %s"
-        if expected_warning is None:
-            warnings.warn(WARNS_NONE_ARG, stacklevel=4)
-            expected_warning_tup = None
-        elif isinstance(expected_warning, tuple):
+        if isinstance(expected_warning, tuple):
             for exc in expected_warning:
                 if not issubclass(exc, Warning):
                     raise TypeError(msg % type(exc))
             expected_warning_tup = expected_warning
-        elif issubclass(expected_warning, Warning):
+        elif isinstance(expected_warning, type) and issubclass(
+            expected_warning, Warning
+        ):
             expected_warning_tup = (expected_warning,)
         else:
             raise TypeError(msg % type(expected_warning))
@@ -301,18 +292,14 @@ class WarningsChecker(WarningsRecorder):
 
     def __exit__(
         self,
-        exc_type: Optional[Type[BaseException]],
-        exc_val: Optional[BaseException],
-        exc_tb: Optional[TracebackType],
+        exc_type: type[BaseException] | None,
+        exc_val: BaseException | None,
+        exc_tb: TracebackType | None,
     ) -> None:
         super().__exit__(exc_type, exc_val, exc_tb)
 
         __tracebackhide__ = True
 
-        if self.expected_warning is None:
-            # nothing to do in this deprecated case, see WARNS_NONE_ARG above
-            return
-
         # BaseExceptions like pytest.{skip,fail,xfail,exit} or Ctrl-C within
         # pytest.warns should *not* trigger "DID NOT WARN" and get suppressed
         # when the warning doesn't happen. Control-flow exceptions should always
@@ -324,7 +311,7 @@ class WarningsChecker(WarningsRecorder):
         ):
             return
 
-        def found_str():
+        def found_str() -> str:
             return pformat([record.message for record in self], indent=2)
 
         try:
@@ -351,3 +338,30 @@ class WarningsChecker(WarningsRecorder):
                         module=w.__module__,
                         source=w.source,
                     )
+
+            # Currently in Python it is possible to pass other types than an
+            # `str` message when creating `Warning` instances, however this
+            # causes an exception when :func:`warnings.filterwarnings` is used
+            # to filter those warnings. See
+            # https://github.com/python/cpython/issues/103577 for a discussion.
+            # While this can be considered a bug in CPython, we put guards in
+            # pytest as the error message produced without this check in place
+            # is confusing (#10865).
+            for w in self:
+                if type(w.message) is not UserWarning:
+                    # If the warning was of an incorrect type then `warnings.warn()`
+                    # creates a UserWarning. Any other warning must have been specified
+                    # explicitly.
+                    continue
+                if not w.message.args:
+                    # UserWarning() without arguments must have been specified explicitly.
+                    continue
+                msg = w.message.args[0]
+                if isinstance(msg, str):
+                    continue
+                # It's possible that UserWarning was explicitly specified, and
+                # its first argument was not a string. But that case can't be
+                # distinguished from an invalid type.
+                raise TypeError(
+                    f"Warning must be str or Warning, got {msg!r} (type {type(msg).__name__})"
+                )
