@@ -253,8 +253,8 @@ class TestRaises:
         msg = "with base 16"
         expr = (
             "Regex pattern did not match.\n"
-            f" Regex: {msg!r}\n"
-            " Input: \"invalid literal for int() with base 10: 'asdf'\""
+            f"  Expected regex: {msg!r}\n"
+            f"  Actual message: \"invalid literal for int() with base 10: 'asdf'\""
         )
         with pytest.raises(AssertionError, match="^" + re.escape(expr) + "$"):
             with pytest.raises(ValueError, match=msg):
@@ -289,7 +289,10 @@ class TestRaises:
             with pytest.raises(AssertionError, match="'foo"):
                 raise AssertionError("'bar")
         (msg,) = excinfo.value.args
-        assert msg == '''Regex pattern did not match.\n Regex: "'foo"\n Input: "'bar"'''
+        assert (
+            msg
+            == '''Regex pattern did not match.\n  Expected regex: "'foo"\n  Actual message: "'bar"'''
+        )
 
     def test_match_failure_exact_string_message(self):
         message = "Oh here is a message with (42) numbers in parameters"
@@ -299,8 +302,8 @@ class TestRaises:
         (msg,) = excinfo.value.args
         assert msg == (
             "Regex pattern did not match.\n"
-            " Regex: 'Oh here is a message with (42) numbers in parameters'\n"
-            " Input: 'Oh here is a message with (42) numbers in parameters'\n"
+            "  Expected regex: 'Oh here is a message with (42) numbers in parameters'\n"
+            "  Actual message: 'Oh here is a message with (42) numbers in parameters'\n"
             " Did you mean to `re.escape()` the regex?"
         )
 
@@ -364,9 +367,7 @@ class TestRaises:
     def test_expected_exception_is_not_a_baseexception(self) -> None:
         with pytest.raises(
             TypeError,
-            match=wrap_escape(
-                "expected exception must be a BaseException type, not 'str'"
-            ),
+            match=wrap_escape("Expected a BaseException type, but got 'str'"),
         ):
             with pytest.raises("hello"):  # type: ignore[call-overload]
                 pass  # pragma: no cover
@@ -377,7 +378,7 @@ class TestRaises:
         with pytest.raises(
             ValueError,
             match=wrap_escape(
-                "expected exception must be a BaseException type, not 'NotAnException'"
+                "Expected a BaseException type, but got 'NotAnException'"
             ),
         ):
             with pytest.raises(NotAnException):  # type: ignore[type-var]
@@ -385,9 +386,7 @@ class TestRaises:
 
         with pytest.raises(
             TypeError,
-            match=wrap_escape(
-                "expected exception must be a BaseException type, not 'str'"
-            ),
+            match=wrap_escape("Expected a BaseException type, but got 'str'"),
         ):
             with pytest.raises(("hello", NotAnException)):  # type: ignore[arg-type]
                 pass  # pragma: no cover
@@ -395,8 +394,8 @@ class TestRaises:
     def test_issue_11872(self) -> None:
         """Regression test for #11872.
 
-        urllib.error.HTTPError on Python<=3.9 raises KeyError instead of
-        AttributeError on invalid attribute access.
+        urllib.error.HTTPError on some Python 3.10/11 minor releases raises
+        KeyError instead of AttributeError on invalid attribute access.
 
         https://github.com/python/cpython/issues/98778
         """
